functor
import
   OS
export
   Convert
   RandBetween 
   Skipper
   FindFeat
   SumarizeMethod
   SumarizeFeature
   AllMeth
   MapFeature
   FindMethod
   MapMethod
   FilterMethod
   PartitionMethod
   Insert
   ListSubtract
   SubSet
   ListAdd
   TakeDet
   TakeDetGC
   Assert
   TripleWaitOr
   Min
   GetDistance
define
   fun{AddLast M Add}
      {Record.adjoinAt M {Length {Filter {Record.arity M} IsNumber}}+1 Add}
   end
   

   %% Converts a stateles structure to a virtual string
   %% Does not corrupt virtualstrings.
   fun{Convert M}
      if {VirtualString.is M} then M
      else {Value.toVirtualString M 10 10} end
   end

   %% Returns a random int between two 
   %% intervals.
   fun{RandBetween X Y}
      ({OS.rand} mod (Y-X+1))+X
   end

   %% Takes one arg and skips, usefull
   %% when the compiler creates erronous code.
   proc{Skipper U} skip end
   
   %% Finds an element in a list that 
   %% has a feature that equals the key
   %%
   %%  ex: {FindFeat [r(id:1) r(id:2) r(id:3)] id 2} = r(id:2)
   fun{FindFeat L Feat Val}
      CC = {NewCell none} in
      {ForAll L proc{$ X}
		   if X.Feat == Val then
		      {Assign CC X}
		   end
		end}
      if {Access CC} == none then raise notFound(Val) end end
      {Access CC}
   end

   %% Finds an object in a list of objects
   %% that has a method of arity one that
   %% equals the Val 
   fun{FindMethod L M Val}
      CC = {NewCell none} in
      {ForAll L proc{$ X}
		   Ans in
		   {X {AddLast M Ans}}
		   if Ans == Val then
		      {Assign CC X}
		   end
		end}
      if {Access CC} == none then raise notFound(Val) end end
      {Access CC}
   end

   %% Summarize the application of the one arity
   %% method Meth on all objects in the list L
   fun{SumarizeMethod L Meth}
      {List.foldR L fun{$ In Old} {In Meth($)} + Old end 0}
   end

   %% Summarize the feature values of all records
   %% in the list L
   fun{SumarizeFeature L Feat}
      {List.foldR L fun{$ In Old} In.Feat + Old end 0}
   end
   
   %% Apply the the method Meth on all
   %% objects in the list L
   proc{AllMeth L Meth}
      {ForAll L proc{$ U} {U Meth} end}
   end

   %% Returns a list with the result
   %% of applying the one arity method Meth
   %% on all objects in the list L 
   fun{MapMethod L Meth}
      {Map L fun{$ H} {H Meth($)} end}
   end

   %% Returns a list of featura access Feat
   %% on all records in list L 
   fun{MapFeature L Feat}
      {Map L fun{$ H} H.Feat end}
   end

   %% Returns a list with the objects
   %% that return true on application of
   %% method Meth
   fun{FilterMethod L Meth}
      {Filter L proc{$ H Ans} {H {AddLast Meth Ans}} end}
   end
   % Corresponding partition
   proc{PartitionMethod L Meth Xs Ys}
      {List.partition L proc{$ H Ans} {H {AddLast Meth Ans}} end Xs Ys}
   end

   %% Inserts an element in a sorted list keeping the list
   %% sorted. Assumes the elements can be compared with <.
   fun{Insert SLs El}
      case SLs of nil then
	 El|nil
      [] S1|More then
	 if El < S1 then
	    El|SLs
	 else
	    S1|{Insert More El}
	 end
      end
   end
 
   %% Subtracts two lists
   %% i.e: {ListSubtract [a b c d] [a c]} = [b d]
   %% Equality on whole elements.
   
   fun{ListSubtract L1 L2}
      {Filter L1 fun{$ L} {Not {Member L L2}} end}
   end

   fun{ListAdd L1 L2}
      D = {NewWeakDictionary _}in
      {ForAll [L1 L2] proc{$ L}
			 {ForAll L proc{$ E} D.E:=true end}
		      end}
      {WeakDictionary.keys D}
   end
   
   fun{SubSet L1 L2}
      {Some L1 fun{$ L} {Member L L2} end}
   end
   
   % Extracts the determinated part (D) of an undetermined list (S), and
   % leaves the undetermined part (R)
   proc{TakeDet S R D}
      if {IsDet S} then
	 D = S.1|{TakeDet S.2 R}
      else
	 D = nil
	 R = S
      end
   end

   % Special version for gc streams containing the gc#<N> token
   proc{TakeDetGC S R D}
      if {IsDet S} then
	 case S.1 of gc#_ then
	    D = {TakeDetGC S.2 R}
	 else
	    D = S.1|{TakeDetGC S.2 R}
	 end
      else
	 D = nil
	 R = S
      end
   end

   % For debugging. Use {Assert Test} or {Assert Test#Msg} to get an
   % exception whenever Test==false.
   proc{Assert V}
      case V of Test#Msg then
	 if Test \= true then raise assertion(Msg) end end
      [] Test then
	 if Test \= true then raise assertion(Test) end end
      else
	 skip
      end
   end

   proc{TripleWaitOr A B C}
      local Trig in
	 thread {Thread.setPriority {Thread.this} high} {WaitOr A B}
	    Trig=unit end
	 thread {Thread.setPriority {Thread.this} high} {Wait C} Trig=unit end
	 {Wait Trig}
      end
   end

   fun{Min X Y}
      if X<Y then X else Y end
   end

   % Calculates the distance between point 1 (X1#Y1) and 2 (X2#Y2)
   % Coordinates given as ints, distance returned as int
   fun{GetDistance X1#Y1 X2#Y2}
      DX=X1-X2
      DY=Y1-Y2
   in
      {FloatToInt {Sqrt {IntToFloat DX*DX+DY*DY}}}
   end
end
 
